"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateCacheFileHash = exports.isChildCompilerRunner = exports.shouldProcessFileByPlugins = exports.getRelativePathToSrcPaths = exports.parseJsonLike = exports.readJsonLike = exports.pathWithoutExtname = exports.extsToGlobPatterns = exports.loadUserTsCompilerOptions = exports.getTsConfigFile = void 0;
const tslib_1 = require("tslib");
const utils_1 = require("@morjs/utils");
const crypto_1 = tslib_1.__importDefault(require("crypto"));
const path_1 = tslib_1.__importDefault(require("path"));
const constants_1 = require("./constants");
/**
 * 提供用户自定义 tsconfig 支持
 * 允许外部获取 ts 配置文件地址
 */
let TS_CONFIG_FILE;
/**
 * 获取用户 cwd 下的 tsconifg 文件地址
 */
function getTsConfigFile() {
    if (TS_CONFIG_FILE)
        return TS_CONFIG_FILE;
    TS_CONFIG_FILE = utils_1.typescript.findConfigFile(utils_1.mor.config.cwd, utils_1.typescript.sys.fileExists);
    return TS_CONFIG_FILE;
}
exports.getTsConfigFile = getTsConfigFile;
/**
 * 保存
 */
let USER_COMPILER_OPTIONS;
/**
 * 获取 ts 配置的 compilerOptions
 * @returns ts 配置的 compilerOptions
 */
function loadUserTsCompilerOptions() {
    var _a;
    if (USER_COMPILER_OPTIONS)
        return USER_COMPILER_OPTIONS;
    const tsConfigFile = getTsConfigFile();
    if (tsConfigFile) {
        let error;
        try {
            const tsConfig = utils_1.typescript.readConfigFile(tsConfigFile, utils_1.typescript.sys.readFile);
            if (tsConfig === null || tsConfig === void 0 ? void 0 : tsConfig.error) {
                error = tsConfig.error;
            }
            else {
                USER_COMPILER_OPTIONS = ((_a = tsConfig === null || tsConfig === void 0 ? void 0 : tsConfig.config) === null || _a === void 0 ? void 0 : _a.compilerOptions) || {};
            }
        }
        catch (err) {
            error = err;
        }
        if (error)
            utils_1.logger.warn(`读取用户 tsconfig.json 失败: ${error}`);
    }
    return USER_COMPILER_OPTIONS || {};
}
exports.loadUserTsCompilerOptions = loadUserTsCompilerOptions;
// 文件名后缀转换为 glob pattern
function extsToGlobPatterns(exts) {
    if (!exts || exts.length === 0)
        return '';
    if (exts.length === 1)
        return `*${exts[0]}`;
    return `*.{${exts.map((ext) => ext.slice(1)).join(',')}}`;
}
exports.extsToGlobPatterns = extsToGlobPatterns;
/**
 * 获取不包含后缀名的文件路径
 * @param filePath - 文件路径
 * @returns 不包含后缀名的文件路径
 */
function pathWithoutExtname(filePath) {
    const extname = path_1.default.extname(filePath);
    // 如果后缀名不存在，或者不是有效的后缀名(AllFileTypes 中声明的) 则不处理
    if (!extname)
        return filePath;
    if (!constants_1.AllFileTypes.includes(extname))
        return filePath;
    // 避免 ./ 在后续操作中丢失
    const prefix = filePath.startsWith('./')
        ? './'
        : filePath.startsWith('.\\')
            ? '.\\'
            : '';
    return (prefix + path_1.default.join(path_1.default.dirname(filePath), path_1.default.basename(filePath, extname)));
}
exports.pathWithoutExtname = pathWithoutExtname;
/**
 * 读取类 JSON 文件
 * 支持 json、 jsonc、json5 三种格式
 * @param fs - fs 实例
 * @param filePath - 类 json 文件
 * @returns json 文件内容
 */
async function readJsonLike(fs, filePath) {
    const fileContent = (await fs.readFile(filePath)).toString('utf-8');
    return parseJsonLike(fileContent, path_1.default.extname(filePath));
}
exports.readJsonLike = readJsonLike;
/**
 * 解析类 JSON 内容
 * 支持 json、 jsonc、json5 三种格式
 * @param content - 文件内容
 * @param extname - 后缀名
 * @returns JSON 对象
 */
function parseJsonLike(content, extname) {
    switch (extname) {
        case '.json':
            return JSON.parse(content);
        case '.jsonc':
            return utils_1.jsoncParser.parse(content);
        case '.json5':
            return utils_1.json5.parse(content);
        default:
            throw new Error(`不支持的 JSON 文件格式: ${extname}`);
    }
}
exports.parseJsonLike = parseJsonLike;
/**
 * 获取相对于 srcPaths 的路径
 *  - 优先匹配 包含 srcPath 的相对路径
 *  - 如果没有包含的，则使用第一个 srcPath 计算 相对路径
 * @param filePath - 文件完整路径
 * @param srcPaths - 自定义 srcPaths 目录
 * @param showWarning - 是否显示告警信息
 * @returns 相对路径
 */
function getRelativePathToSrcPaths(filePath, srcPaths, showWarning = true) {
    const roots = (srcPaths === null || srcPaths === void 0 ? void 0 : srcPaths.length) ? srcPaths : [];
    for (const srcPath of roots) {
        if (filePath.startsWith(srcPath)) {
            return path_1.default.relative(srcPath, filePath);
        }
    }
    // 如果上面的逻辑未命中, 则代表某处引用的文件超出了源代码之外, 这里提示警告
    // 并约束该文件路径, 防止发生对源代码的意外修改
    // 例 ../../abc/index.js => pdir/pdir/abc/index.js
    // pdir 代表 parent directory 父级目录
    let relativePath = path_1.default.relative(roots[0], filePath);
    if (relativePath.startsWith('..')) {
        relativePath = relativePath.replace(/\.\./g, 'pdir');
    }
    if (showWarning) {
        utils_1.logger.warnOnce(`引用的文件: ${filePath} 不在约定的源码目录中\n` +
            `路径优化为: ${relativePath} \n` +
            `其中: pdir 代表 parent direcotry, 用于替换 .. 路径`);
    }
    return relativePath;
}
exports.getRelativePathToSrcPaths = getRelativePathToSrcPaths;
/**
 * 判断是否需要插件处理文件，用于 loader 中的逻辑判断
 * 1. 非 node_modules 文件总是处理
 * 2. node_modules 文件基于逻辑判断是否处理
 * @param filePath - 文件路径
 * @param processNodeModules - 强制处理
 * @param byDefault - 默认操作, 默认为 false
 */
function shouldProcessFileByPlugins(filePath, processNodeModules, byDefault = false) {
    if (processNodeModules === true)
        return true;
    if (constants_1.NODE_MODULE_REGEXP.test(filePath)) {
        if (processNodeModules === false)
            return false;
        if (processNodeModules != null) {
            // 如果配置了 include 或 exclude 逐一校验
            // exclude 优先级高于 include
            const exclude = (0, utils_1.asArray)(processNodeModules.exclude || []);
            const include = (0, utils_1.asArray)(processNodeModules.include || []);
            if (exclude.length &&
                exclude.reduce((res, e) => e.test(filePath) || res, false)) {
                return false;
            }
            if (include.length) {
                if (include.some((e) => (e.test(filePath) ? true : undefined)))
                    return true;
            }
            // include 未配置, 则默认为 true
            else {
                return true;
            }
        }
        return byDefault;
    }
    // 普通文件固定返回 true
    return true;
}
exports.shouldProcessFileByPlugins = shouldProcessFileByPlugins;
/**
 * 判断当前 runner 是否为 编译子 runner
 * @param runner - Runner 实例
 * @returns `true` or `false`
 */
function isChildCompilerRunner(runner) {
    return !!runner.context.get(constants_1.CHILD_COMPILER_RUNNER);
}
exports.isChildCompilerRunner = isChildCompilerRunner;
/**
 * 基于用户配置生成缓存文件 hash
 * @param userConfig - 用户配置
 * @param prefixKeys - 缓存 key 前缀
 * @returns 生成的 cache hash
 */
function generateCacheFileHash(userConfig, prefixKeys = []) {
    // 这里综合考虑可能会影响到 webpack 缓存以及 entries 缓存的情况
    const hash = crypto_1.default
        .createHash('md5')
        .update(JSON.stringify({
        minimize: userConfig.minimize,
        jsMinimizer: userConfig.jsMinimizer,
        jsMinimizerOptions: userConfig.jsMinimizerOptions,
        cssMinimizer: userConfig.cssMinimizer,
        cssMinimizerOptions: userConfig.cssMinimizerOptions,
        xmlMinimizer: userConfig.xmlMinimizer,
        xmlMinimizerOptions: userConfig.xmlMinimizerOptions,
        externals: String(userConfig.externals).valueOf(),
        shared: userConfig.shared,
        consumes: userConfig.consumes,
        srcPath: userConfig.srcPath,
        srcPaths: userConfig.srcPaths,
        ignore: userConfig.ignore,
        compilerOptions: userConfig.compilerOptions,
        conditionalCompile: userConfig.conditionalCompile,
        define: userConfig.define,
        alias: userConfig.alias,
        processPlaceholderComponents: userConfig.processPlaceholderComponents,
        modules: (0, utils_1.asArray)(userConfig.modules || []).filter((m) => (m === null || m === void 0 ? void 0 : m.mode) === 'compile'),
        customEntries: userConfig.customEntries
    }))
        .digest('hex');
    const configName = (userConfig.name || '').replace(/[^a-zA-Z0-9-_]/g, '_');
    return prefixKeys
        .concat([
        configName,
        userConfig.compileType,
        userConfig.target,
        userConfig.compileMode,
        hash
    ])
        .join('-');
}
exports.generateCacheFileHash = generateCacheFileHash;
//# sourceMappingURL=utils.js.map